@@ -46,6 +46,7 @@ RemoteControl::RemoteControl(QObject *parent) :
     rds_status = false;
     signal_level = -200.0;
     squelch_level = -150.0;
+    audio_gain = -6.0;
     audio_recorder_status = false;
     receiver_running = false;
     hamlib_compatible = false;
@@ -244,8 +245,12 @@ void RemoteControl::startRead()
         answer = cmd_LOS();
     else if (cmd == "LNB_LO")
         answer = cmd_lnb_lo(cmdlist);
+    else if (cmd == "\\chk_vfo")
+        answer = QString("0\n");
     else if (cmd == "\\dump_state")
         answer = cmd_dump_state();
+    else if (cmd == "\\get_powerstat")
+        answer = QString("1\n");
     else if (cmd == "q" || cmd == "Q")
     {
         // FIXME: for now we assume 'close' command
@@ -351,6 +356,12 @@ void RemoteControl::setSquelchLevel(double level)
     squelch_level = level;
 }
 
+/*! \brief Set audio gain (from mainwindow). */
+void RemoteControl::setAudioGain(float gain)
+{
+    audio_gain = gain;
+}
+
 /*! \brief Start audio recorder (from mainwindow). */
 void RemoteControl::startAudioRecorder(QString unused)
 {
@@ -620,15 +631,19 @@ QString RemoteControl::cmd_get_level(QStringList cmdlist)
         QStringList names;
         for(auto &g : gains)
             names.push_back(QString("%1_GAIN").arg(QString::fromStdString(g.name)));
-        answer = QString("SQL STRENGTH %1\n").arg(names.join(" "));
+        answer = QString("SQL STRENGTH AF %1\n").arg(names.join(" "));
     }
     else if (lvl.compare("STRENGTH", Qt::CaseInsensitive) == 0 || lvl.isEmpty())
     {
-        answer = QString("%1\n").arg(signal_level, 0, 'f', 1);
+        answer = QString("%1\n").arg((double)signal_level, 0, 'f', 1);
     }
     else if (lvl.compare("SQL", Qt::CaseInsensitive) == 0)
     {
-        answer = QString("%1\n").arg(squelch_level, 0, 'f', 1);
+        answer = QString("%1\n").arg((double)squelch_level, 0, 'f', 1);
+    }
+    else if (lvl.compare("AF", Qt::CaseInsensitive) == 0)
+    {
+        answer = QString("%1\n").arg((double)audio_gain, 0, 'f', 1);
     }
     else if (lvl.endsWith("_GAIN"))
     {
@@ -662,7 +677,7 @@ QString RemoteControl::cmd_set_level(QStringList cmdlist)
         QStringList names;
         for(auto &g : gains)
             names.push_back(QString("%1_GAIN").arg(QString::fromStdString(g.name)));
-        answer = QString("SQL %1\n").arg(names.join(" "));
+        answer = QString("SQL AF %1\n").arg(names.join(" "));
     }
     else if (lvl.compare("SQL", Qt::CaseInsensitive) == 0)
     {
@@ -679,6 +694,21 @@ QString RemoteControl::cmd_set_level(QStringList cmdlist)
             answer = QString("RPRT 1\n");
         }
     }
+    else if (lvl.compare("AF", Qt::CaseInsensitive) == 0)
+    {
+        bool ok;
+        float new_audio_gain = cmdlist.value(2, "ERR").toFloat(&ok);
+        if (ok)
+        {
+            answer = QString("RPRT 0\n");
+            new_audio_gain = std::max<float>(-80.0f, std::min<float>(50.0f, new_audio_gain));
+            emit newAudioGain(new_audio_gain);
+        }
+        else
+        {
+            answer = QString("RPRT 1\n");
+        }
+    }
     else if (lvl.endsWith("_GAIN"))
     {
         lvl.chop(5);
