@@ -26,15 +26,10 @@
 #include <gnuradio/gr_complex.h>
 #include <gnuradio/blocks/complex_to_float.h>
 #include <gnuradio/blocks/float_to_complex.h>
+#include <gnuradio/blocks/sub.h>
 #include <gnuradio/hier_block2.h>
 #include <gnuradio/filter/single_pole_iir_filter_cc.h>
 
-#if GNURADIO_VERSION < 0x030800
-#include <gnuradio/blocks/sub_cc.h>
-#else
-#include <gnuradio/blocks/sub.h>
-#endif
-
 class dc_corr_cc;
 class iq_swap_cc;
 
@@ -87,7 +82,7 @@ iq_swap_cc_sptr make_iq_swap_cc(bool enabled);
 /*! \brief Block to swap I and Q channels.
  *  \ingroup DSP
  */
-class iq_swap_cc : public gr::hier_block2
+class iq_swap_cc : public gr::sync_block
 {
     friend iq_swap_cc_sptr make_iq_swap_cc(bool enabled);
 
@@ -97,10 +92,11 @@ protected:
 public:
     ~iq_swap_cc();
     void set_enabled(bool enabled);
+    int work(int noutput_items,
+             gr_vector_const_void_star& input_items,
+             gr_vector_void_star& output_items);
 
 private:
-    gr::blocks::complex_to_float::sptr d_c2f;
-    gr::blocks::float_to_complex::sptr d_f2c;
     bool d_enabled;
 };
 
