@@ -54,6 +54,7 @@ void DockRDS::updateRDS(QString text, int type)
     switch (type)
     {
     case 0:
+        emit rdsPI(text);
         ui->program_information->setText(text);
         break;
     case 1:
@@ -136,3 +137,12 @@ void DockRDS::on_rdsCheckbox_toggled(bool checked)
 {
     emit rdsDecoderToggled(checked);
 }
+
+/* used by remote control */
+void DockRDS::setRDSmode(bool cmd)
+{
+    if (!ui->rdsCheckbox->isEnabled())
+        return;
+
+    ui->rdsCheckbox->setChecked(cmd);
+}
