@@ -33,7 +33,6 @@
 
 #include <QtGui>
 #include <QFrame>
-#include <QImage>
 
 class CMeter : public QFrame
 {
@@ -41,39 +40,22 @@ class CMeter : public QFrame
 
 public:
     explicit CMeter(QWidget *parent = 0);
-    explicit CMeter(float min_level = -100.0, float max_level = 10.0,
-                    QWidget *parent = 0);
     ~CMeter();
 
     QSize minimumSizeHint() const;
     QSize sizeHint() const;
 
-    void setMin(float min_level);
-    void setMax(float max_level);
-    void setRange(float min_level, float max_level);
-
-    void draw();
-    void UpdateOverlay(){DrawOverlay();}
-
 public slots:
     void setLevel(float dbfs);
     void setSqlLevel(float dbfs);
 
 protected:
     void paintEvent(QPaintEvent *event);
-    void resizeEvent(QResizeEvent* event);
 
 private:
-    void DrawOverlay();
+    void draw(QPainter &painter);
+    void drawOverlay(QPainter &painter);
 
-    QFont   m_Font;
-    QPixmap m_2DPixmap;
-    QPixmap m_OverlayPixmap;
-    QSize   m_Size;
-    QString m_Str;
-    qreal   m_pixperdb;     // pixels / dB
-    qreal   m_Siglevel;
     float   m_dBFS;
-    qreal   m_Sql;
-    qreal   m_SqlLevel;
+    float   m_Sql;
 };
